﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace xamloomba
{
	public partial class Loomba : UserControl
	{
		public Loomba()
		{
			// 変数を初期化するときに必要となります
			InitializeComponent();
		}

        private void StopCurrentAction()
        {
            this.turnLeft.Pause();
            this.turnRight.Pause();
        }

        private const int SpeedPerSec = 100;
        private const int SpeedTurn = 2;


        /// <summary>
        /// 指定位置に移動します
        /// </summary>
        /// <param name="dest"></param>
        public void Move(Point dest)
        {
            this.StopCurrentAction();
            Point original = this.GetOriginalPosition();
            Point current = this.GetCurrentPosition();
            double distance = this.GetDistanceTo(dest);
            double angle = this.GetAngleTo(dest);

            //回転に要する時間
            TimeSpan turnTime = TimeSpan.FromSeconds(Math.Abs(this.GetCurrentAngle() - angle) * 2 / 360);
            

            //目的地までの所要時間
            TimeSpan time = TimeSpan.FromSeconds(distance / SpeedPerSec);

            //ストーリーボードを作成
            Storyboard sb = new Storyboard();

            //ストーリーボードの所要時間を設定
            sb.Duration = time.Add(turnTime);

            //回転用のアニメーション
            DoubleAnimation turnAnimation = new DoubleAnimation();
            turnAnimation.To = angle;
            turnAnimation.Duration = new Duration(turnTime);
            sb.Children.Add(turnAnimation);

            Storyboard.SetTarget(turnAnimation, this.loomba);
            Storyboard.SetTargetProperty(turnAnimation, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[2].(RotateTransform.Angle)"));


            //縦方向移動アニメーション
            DoubleAnimationUsingKeyFrames topAnimation = new DoubleAnimationUsingKeyFrames();
            topAnimation.BeginTime = turnTime;  //回転終了後に開始
            SplineDoubleKeyFrame topKf = new SplineDoubleKeyFrame();
            topKf.KeyTime = KeyTime.FromTimeSpan(time);
            topKf.Value = dest.Y - original.Y;
            topAnimation.KeyFrames.Add(topKf);
            sb.Children.Add(topAnimation);

            Storyboard.SetTarget(topAnimation, this.loomba);
            Storyboard.SetTargetProperty(topAnimation, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[3].(TranslateTransform.Y)"));

            //横方向移動アニメーション
            DoubleAnimationUsingKeyFrames leftAnimation = new DoubleAnimationUsingKeyFrames();
            leftAnimation.BeginTime = turnTime; //回転終了後に開始
            SplineDoubleKeyFrame leftKf = new SplineDoubleKeyFrame();
            leftKf.KeyTime = KeyTime.FromTimeSpan(time);
            leftKf.Value = dest.X - original.X;
            leftAnimation.KeyFrames.Add(leftKf);
            sb.Children.Add(leftAnimation);

            Storyboard.SetTarget(leftAnimation, this.loomba);
            Storyboard.SetTargetProperty(leftAnimation, new PropertyPath("(UIElement.RenderTransform).(TransformGroup.Children)[3].(TranslateTransform.X)"));

            //ストーリーボードを実行
            sb.Begin();
            
        }


        private Point GetOriginalPosition()
        {
            Point original = new Point(Canvas.GetLeft(this), Canvas.GetTop(this));
            return original;
        }

        private Point GetCurrentPosition()
        {
            Point original = this.GetOriginalPosition();

            TranslateTransform trans = ((TransformGroup)loomba.RenderTransform).Children[3] as TranslateTransform;
            Point current = new Point(original.X + trans.X, original.Y + trans.Y);
            return current;
        }

        private double GetCurrentAngle()
        {
            RotateTransform trans = ((TransformGroup)loomba.RenderTransform).Children[2] as RotateTransform;

            return trans.Angle;
        }

        private double GetDistanceTo(Point dest)
        {
            Point current = this.GetCurrentPosition();
            double dx = current.X - dest.X;
            double dy = current.Y - dest.Y;

            return Math.Sqrt(dx * dx + dy * dy);
        }

        private double GetAngleTo(Point dest)
        {
            Point current = this.GetCurrentPosition();
            double dx = current.X - dest.X;
            double dy = current.Y - dest.Y;

            return Math.Atan2(dy, dx) * 180 / Math.PI - 90;
        }

	}
}